/**
 * A demo of a cloth falling on a cube box, to get started with soft bodies.
 */

#include "demo_skeleton.h"
#include "softbody_mesh.h"

#include <BulletSoftBody/btSoftBody.h>
#include <BulletSoftBody/btSoftBodyHelpers.h>

class EmptyBag : public DemoSkeleton {
public:
	EmptyBag();

protected:
	void setupScene() override;
	void renderIteration() override;

private:
	btSoftBody *mBag = nullptr;
	SoftbodyMesh mBagMesh;
};

EmptyBag::EmptyBag()
{
	mShowPhysicsDebug = false;
}

void EmptyBag::setupScene()
{
	createGroundPlane();

	{
		btTransform t;
		t.setIdentity();
		t.setOrigin(btVector3(0, 10, 0));
		mBag = mBagMesh.createFromFile("res/bag.ply", mSoftBodyWorldInfo, t);
		mBag->getCollisionShape()->setMargin(0.001f);
		mBag->generateBendingConstraints(2, mBag->appendMaterial());
		mBag->setTotalMass(0.1);
		mBag->m_cfg.piterations = 5;
		mBag->m_cfg.kDP = 0.005f;
		static_cast<btSoftRigidDynamicsWorld *>(mWorld)->addSoftBody(mBag);

		mBagMesh.createModel();
	}
}

void EmptyBag::renderIteration()
{
	DrawGrid(20, 1.0f);
	DrawModel(mBagMesh.updateModel(), { 0, 0, 0 }, 1.0, BLUE);
}

int main()
{
	EmptyBag().run();
	return 0;
}
