/**
 * A simple demo of a cube falling just to get started.
 */

#include "demo_skeleton.h"

class FallingCube : public DemoSkeleton {
protected:
	/// Setup physics only for rigid bodies
	void setupPhysics() override;

	/// Create the plane and the cube
	void setupScene() override;

	/// Draw the cube and the grid
	void renderIteration() override;

private:
	Model mCube;
	btDefaultMotionState *mCubeMotionState = nullptr;
};

void FallingCube::setupPhysics()
{
	mCollisionConfiguration = new btDefaultCollisionConfiguration();
	mDispatcher = new btCollisionDispatcher(mCollisionConfiguration);
	mBroadphase = new btDbvtBroadphase();
	mSolver = new btSequentialImpulseConstraintSolver();
	mWorld = new btDiscreteDynamicsWorld(
		mDispatcher, mBroadphase, mSolver, mCollisionConfiguration);
	mWorld->setGravity({ 0, -9.81, 0 });
}

void FallingCube::setupScene()
{
	createGroundPlane(0.0f);

	const float cubeSize = 2.0f;
	Mesh cubeMesh = GenMeshCube(cubeSize, cubeSize, cubeSize);
	mCube = LoadModelFromMesh(cubeMesh);
	// Remember: Bullet takes half length
	btCollisionShape *cubeShape = new btBoxShape(
		{ cubeSize / 2, cubeSize / 2, cubeSize / 2 });
	mCollisionShapes.push_back(cubeShape);

	btTransform cubeTransform;
	cubeTransform.setIdentity();
	btQuaternion cubeRotation;
	cubeRotation.setRotation({ 1, 0, 0 }, PI / 6);
	cubeTransform.setRotation(cubeRotation);
	cubeTransform.setOrigin({ 0, 10, 0 });
	transformToMatrix(cubeTransform, mCube.transform);

	btScalar mass(1.f);
	btVector3 localInertia;
	cubeShape->calculateLocalInertia(mass, localInertia);
	mCubeMotionState = new btDefaultMotionState(cubeTransform);
	btRigidBody::btRigidBodyConstructionInfo rbInfo(
		mass, mCubeMotionState, cubeShape, localInertia);

	btRigidBody *body = new btRigidBody(rbInfo);
	mWorld->addRigidBody(body);
}

void FallingCube::renderIteration()
{
	btTransform transform;
	mCubeMotionState->getWorldTransform(transform);
	transformToMatrix(transform, mCube.transform);
	DrawModel(mCube, { 0, 0, 0 }, 1.0, YELLOW);
	DrawModelWires(mCube, { 0, 0, 0 }, 1.0, RED);
	DrawGrid(20, 1.0f);
}

int main()
{
	FallingCube().run();
	return 0;
}
