#include "ray_bt_debug.h"

#include <iostream>

static inline void btToRay(const btVector3 &bt, Vector3 &r)
{
	r = { bt.x(), bt.y(), bt.z() };
}

static inline void btToRay(const btVector3 &bt, Color &r)
{
	r.r = static_cast<float>(bt.x() * 255);
	r.g = static_cast<float>(bt.y() * 255);
	r.b = static_cast<float>(bt.z() * 255);
	r.a = 255;
}

void RayBtDebug::drawFrame()
{
	while (!mLines.empty()) {
		Line &l = mLines.front();
		DrawLine3D(l.from, l.to, l.color);
		mLines.pop();
	}

	for (auto it = mPoints.begin(); it != mPoints.end(); ++it) {
		if (it->expiration < GetTime()) {
			DrawPoint3D(it->position, it->color);
		} else {
			auto remove = it--;
			mPoints.erase(remove);
		}
	}
}

void RayBtDebug::drawTexts(const Camera &camera)
{
	while (!mTexts.empty()) {
		Text &t = mTexts.front();
		Vector2 pos2d = GetWorldToScreen(t.position, camera);
		DrawText(t.text.c_str(), pos2d.x, pos2d.y, 20, LIME);
		mTexts.pop();
	}
}

void RayBtDebug::drawLine(const btVector3 &from, const btVector3 &to,
	const btVector3 &color)
{
	Line l;
	btToRay(from, l.from);
	btToRay(to, l.to);
	btToRay(color, l.color);
	mLines.push(l);
}

void RayBtDebug::drawContactPoint(const btVector3 &pointOnB,
	const btVector3 &normalOnB, btScalar distance, int lifeTime,
	const btVector3& color)
{
	Point p;
	btToRay(pointOnB, p.position);
	btToRay(color, p.color);
	p.expiration = GetTime() + lifeTime;
	mPoints.push_back(p);
}

void RayBtDebug::reportErrorWarning(const char *warningString)
{
	// Usually Bullet already pass the newline
	std::cerr << "[bullet] " << warningString;
}

void RayBtDebug::draw3dText(const btVector3 &location, const char *textString)
{
	/* Not really a 3D text, anyway Bullet does not pass the normal for the
	plane that should contain the text, so this is the best we can do. */
	Text t;
	btToRay(location, t.position);
	t.text = textString;
	mTexts.push(t);
}

void RayBtDebug::setDebugMode(int debugMode)
{
	mDebugMode = debugMode;
}

int RayBtDebug::getDebugMode() const
{
	return mDebugMode;
}
