#include <list>
#include <queue>
#include <string>

#include <raylib.h>

#include <LinearMath/btIDebugDraw.h>

#ifndef ray_bt_debug_ac175c62e49acb1c1f33ddd84772712d
#define ray_bt_debug_ac175c62e49acb1c1f33ddd84772712d

/**
 * A class that draws Bullet debug info using Raylib.
 *
 * The calls to draw lines and points are buffered, i.e. the information of what
 * to draw are saved into data structures, and then are drawn when the drawFrame
 * method is called.
 */
class RayBtDebug : public btIDebugDraw {
public:
	/// Struct to keep in memory the information about a line to draw
	struct Line {
		/// Starting point of the line
		Vector3 from;
		/// Ending point of the line
		Vector3 to;
		/// Color of the line
		Color color;
	};

	/// Struct to keep in memory the information about a point to draw
	struct Point {
		/// The position of the point
		Vector3 position;
		/// The color of the point
		Color color;
		/// The expiration of the point, as a reference to GetTime()
		double expiration;
	};

	/// Struct to keep in memory the information about a text to write
	struct Text {
		/// The position
		Vector3 position;
		/// The string to write
		std::string text;
	};

	/// Draw the debug content in the frame
	void drawFrame();

	/**
	 * Draw the debug texts
	 *
	 * \param camera The active camera (needed to compute the text location)
	 */
	void drawTexts(const Camera &camera);

	void drawLine(const btVector3 &from,const btVector3 &to,
		const btVector3 &color) override;
	void drawContactPoint(const btVector3 &pointOnB, const btVector3 &normalOnB,
		btScalar distance, int lifeTime, const btVector3& color) override;
	void reportErrorWarning(const char *warningString) override;
	void draw3dText(const btVector3 &location, const char *textString) override;
	void setDebugMode(int debugMode) override;
	int getDebugMode() const override;

private:
	/// The debug mode, as passed from Bullet
	int mDebugMode;

	/// The queue of lines to draw
	std::queue<Line, std::list<Line>> mLines;

	/// The list of points to draw
	std::list<Point> mPoints;

	/// The queue of texts to draw
	std::queue<Text, std::list<Text>> mTexts;
};

#endif
