#include <raylib.h>
#include <rlgl.h>

#include <BulletSoftBody/btSoftBody.h>

#include <string>
#include <vector>

#ifndef softbody_mesh_06f884263a9adf160375a5033e1cbc77
#define softbody_mesh_06f884263a9adf160375a5033e1cbc77

class SoftbodyMesh {
public:

	/**
	 * Create a Raylib model and upload it to the GPU.
	 *
	 * This function needs the soft body to be defined.
	 * In particular, it uses the softbody to take the data to populate the mesh.
	 *
	 * \return The created model
	 */
	Model &createModel();

	/**
	 * Update the model and its GPU data using the body data.
	 *
	 * This function takes the data from the soft body, updates the CPU data of
	 * the mesh and then uploads the updated data also to the GPU.
	 * It uploads vertices, normals and indices.
	 *
	 * \return The updated model
	 */
	Model &updateModel();

	/**
	 * Create a softbody from a file.
	 *
	 * The file format must be supported by Raylib.
	 * If the body is not null, this function does not do anything.
	 *
	 * The body is created and assigned to the mBody member, but it is not added
	 * to the world. The user should take the created body, customize its
	 * parameters, and add it to the world.
	 *
	 * \param filename The name of the file
	 * \param worldInfo The object that contains information about the world
	 * \param transform A transform to apply to all the points
	 * \return The created body, or the previous one, if not null
	 */
	btSoftBody *createFromFile(const std::string &filename,
		btSoftBodyWorldInfo &worldInfo,
		const btTransform &transform = btTransform());

	Model mModel;
	btSoftBody *mBody = nullptr;

private:
	void setMeshContent(Mesh &mesh);

	std::vector<float> mVertices;
	std::vector<float> mNormals;
	std::vector<unsigned short> mIndices;
	std::vector<unsigned int> mVbo;
};

#endif
